import os


def find_level(root, startpath):
    return root.replace(startpath, '').count(os.sep)


def print_files(startpath):
    for root, dirs, files in os.walk(startpath):
        level = find_level(root, startpath)
        indent = ' ' * 4 * (level)
        print('{}{}/'.format(indent, os.path.basename(root)))
        subindent = ' ' * 4 * (level + 1)
        for f in files:
            print('{}{}'.format(subindent, f))


def write_file(filepath, string, debug=False):
    with open(filepath, "w", encoding="utf-8") as file:
        file.write(string)

        if debug:
            print(f"File changed: {file.name}")


def insert_file(filepath, string, index, new_index=None, debug=False):
    def get_index():
        return new_index if new_index else index

    def generate_insertion():
        insertion = ""
        if bool(string):
            index = get_index()
            insertion += index + "\n\n"
            insertion += string + "\n"
            insertion += index + "\n"
        return insertion

    filestr = ""
    inserted = False
    if os.path.exists(filepath):
        with open(filepath, "r+", encoding="utf-8") as file:
            save = True

            for line in file:
                if not inserted and index in line.replace(" ", ""):
                    filestr += generate_insertion()
                    save = False
                    inserted = True
                else:
                    if not inserted or save:
                        filestr += line
                    else:
                        if index in line.replace(" ", ""):
                            save = True

    else:
        filestr = generate_insertion()
        inserted = True

    # If no insertion happend, create new section
    if not inserted:
        # BUG: If index is not found in the file while new index used, index string is dublicated
        # WARN: "\n" olmazsa satırın ucuna eklemekte, bu da indexin olduğu satırın kaybolmasından dolayı verinin silinmesine neden olmakta
        new_line_count = 2 - filestr[-2:].count("\n")
        filestr += "\n" * new_line_count
        filestr += generate_insertion()
        inserted = True

    write_file(filepath, filestr, debug=debug)

    if debug:
        print(f"Index: {index} New Index: {new_index}")



import os
from urllib.parse import quote
from .filesystem import insert_file as fs_insert_file


# DEV: Figure out index string in markdown file

def remove_extension(filepath: str) -> str:
    """Dosya uzantısını kaldırma

    Args:
        filepath (str): Dosya yolu

    Returns:
        str: Uzantsız dosya yolu
    """

    filepath, _ = os.path.splitext(filepath)
    return filepath


def barename(path: str) -> str:
    """Dosya veya dizin yolundan, yol ve uzantıyı temizleme

    Args:
        filepath (str): Dosya yolu

    Returns:
        str: Sadece dosya ismi
    """

    pathname = path
    if os.path.isfile(path):
        pathname = remove_extension(path)

    pathname = os.path.basename(pathname)

    return pathname


def relativepath(path: str, root=os.getcwd()) -> str:
    """ Statik yol verisini dinamik yol verisine dönüştürme

    Args:
        pathname (str): Yol ismi

    Returns:
        str: Dönüştürülen metin
    """

    return path.replace(root, '.')


def encodedpath(path: str) -> str:
    """ Verilen yolu url formatında kodlama

    Windows için gelen '\\' karakteri '/' karakterine çevrilir

    Args:
        pathname (str): Yol ismi

    Returns:
        str: Kodlanmış metin
    """

    return quote(path.replace("\\", "/"))


def create_indent(level, size=4):
    return ' ' * size * (level)


def create_link(path: str, header=None, root=os.getcwd()) -> str:
    """Verilen yola uygun kodlanmış markdown linki oluşturma

    Args:
        pathname (str): Yol
        header (str): Link

    Returns:
        str: Oluşturulan link metni
    """

    pathname = barename(path) if not header else header
    path = relativepath(path, root=root)
    path = encodedpath(path)

    link = f"- [{pathname}]({path})\n"
    return link


def create_header(name: str, headerlvl: int) -> str:
    return "#" * headerlvl + f" {name}\n\n"


def read_first_header(filepath):
    header = ""
    with open(filepath, "r", encoding="utf-8") as file:
        for line in file:
            if line.startswith("#"):
                header = line.strip().replace("# ", "")
                break

    if "<!--" in header:
        header = header[:header.index("<!--")].strip()

    return header


def make_comment(string):
    return f"<!--{string}-->"


def insert_file(filepath, string, index, force=False, fileheader=None, new_index=None):
    if force and not os.path.exists(filepath):
        create_markdown_file(filepath, header=fileheader)

    index = make_comment(index)

    if bool(new_index):
        new_index = make_comment(new_index)

    fs_insert_file(filepath, string, index, new_index)


def create_markdown_file(filepath, header=None):
    if not bool(header):
        header = os.path.basename(filepath)

    with open(filepath, "w", encoding="utf-8") as file:
        file.write(create_header(header,  1))



import os
from ypackage.filesystem import find_level
from ypackage.markdown import create_indent, create_link, create_header, read_first_header, insert_file, create_markdown_file, relativepath, encodedpath

SUMMARY_FILE_HEADER = "# Summary"
SUMMARY_FILE = "SUMMARY.md"
README_FILE = "README.md"
CHANGELOG_FILE = "CHANGELOG.md"
CHANGELOG_HEADER = u"👀 Neler değişti"


def create_file_link(path: str):
    path = encodedpath(path)
    return r'{% file src="' + path + r'" %}' + '\n'


def list_workspace(startpath, level_limit: int = -1, privates=[], debug=False):
    # Fixs "Folder\" "Folder" difference. All of them is equel "Folder"
    startpath = os.path.normpath(startpath)

    lines = [SUMMARY_FILE_HEADER + "\n\n"]
    for root, dirs, files in os.walk(startpath):
        # Remove private directories
        dirs[:] = [d for d in dirs if not d.startswith(
            '.') and d not in privates]

        if root == startpath:
            # Find title of repo from README.md
            headpath = os.path.join(root, README_FILE)
            header = read_first_header(headpath)
            header = create_link(headpath, header=header, root=startpath)
            lines.append(header)

            for f in files:
                if f == CHANGELOG_FILE:
                    changelog_path = os.path.join(root, CHANGELOG_FILE)
                    changelog_link = create_link(
                        changelog_path, header=CHANGELOG_HEADER, root=startpath)

                    # Sadece 1 master link olur (indent)
                    level = find_level(root, startpath) + 1
                    indent = create_indent(level, size=2)
                    lines.append('{}{}'.format(indent, changelog_link))

            # Skip the startpath files
            continue

        if debug:
            print(f"\nRoot: {root}\nStartpath: {startpath}")

        level = find_level(root, startpath)
        if level_limit == -1 or level <= level_limit:  # MAX
            indent = create_indent(level, size=2)

            # If there exist README in dir, link it instead folder (like GitHub)
            if README_FILE in files:
                readme_path = os.path.join(root, README_FILE)
                dirlink = create_link(
                    readme_path, header=read_first_header(readme_path), root=startpath)
            else:
                dirlink = create_link(root, root=startpath)

            lines.append('{}{}'.format(indent, dirlink))

            subindent = create_indent(level + 1, size=2)
            for f in files:
                # Only looks markdown files
                if ".md" in f:
                    if "README.md" in f:
                        continue

                    filepath = os.path.join(root, f)
                    lines.append('{}{}'.format(
                        subindent, create_link(filepath, header=read_first_header(filepath), root=startpath)))

    return lines


def generate_summary(inputDir, level_limit: int = -1, privates=[], footer_path=None, index="Index", new_index=None):
    filepath = os.path.join(inputDir, SUMMARY_FILE)
    lines = list_workspace(
        inputDir, level_limit=level_limit, privates=privates)

    filestr = ""
    if footer_path and os.path.isfile(footer_path):
        with open(os.path.join(footer_path), "r", encoding="utf-8") as file:
            filestr = "\n" + file.read()

    filestr = "".join(lines) + filestr
    insert_file(filepath, filestr, index=index, new_index=new_index)


def generate_readmes(startpath, level_limit: int = -1, privates=[], index="Index", header="📂 Harici Dosyalar", new_index=None, clearify=False):
    # DEV: Ders notlarını README'ye ekleme direkt olarak dizin dosya oluştur
    def clear_private_dirs() -> list:
        return [d for d in dirs if not d.startswith(
            '.') and d not in privates]

    def generate_links_for_nonmarkdowns(files) -> str:
        # OLD: Eski yapıdır, kontrollü bir şekilde kaldırılmalı
        filestr = ""
        links = []
        for f in files:
            if not ".md" in f:
                links.append(create_file_link(os.path.join(".", f))

        if bool(links):
            filestr=create_header(header, 2)
            filestr += "".join(links)

        return filestr

    def get_readme_path(root) -> str:
        return os.path.join(root, README_FILE)

    def generate_markdown_files_for_subitems(startpath) -> str:
        # DEV: Aynı dizindekiler aynı dosya altına yazılacak
        for root, _, files in os.walk(startpath):
            if root == startpath:
                continue

            filestr=""
            fileheader=os.path.basename(root)
            filepath=os.path.join(startpath, fileheader + ".md")

            level=find_level(root, startpath)
            link_root=os.path.dirname(root)
            for _ in range(level - 1):
                link_root=os.path.dirname(link_root)

            for f in files:
                subfilepath=os.path.join(root, f)
                if not ".md" in f:
                    filestr += create_file_link(subfilepath)
                else:
                    filestr += create_link(subfilepath, root=link_root)

            if bool(filestr):
                insert_file(filepath, filestr, index=index,
                            force=True, fileheader=fileheader, new_index=new_index)

    filestr=""
    for root, dirs, files in os.walk(startpath):
        # Remove private directories
        dirs[:]=clear_private_dirs()

        # Skip startpath
        if root == startpath:
            continue

        filestr=""
        level=find_level(root, startpath)
        if level_limit == -1 or level < level_limit:
            filestr=generate_links_for_nonmarkdowns(files)
        elif level == level_limit:  # 1
            generate_markdown_files_for_subitems(root)
        else:
            if clearify and level > level_limit:
                readme_path=get_readme_path(root)
                if os.path.exists(readme_path):
                    os.remove(readme_path)
            continue

        if bool(filestr):
            filepath=get_readme_path(root)
            fileheader=os.path.basename(root)
            insert_file(filepath, filestr, index=index,
                        force=True, fileheader=fileheader, new_index=new_index)



def exit_if_not(condition, message=None):
    if not condition:
        exit(message)


def list_difference(list1: list, list2: list, safe: bool = True) -> list:
    """İki listenin farkını alır

    Arguments:
        list1 {list} -- Ana liste
        list2 {list} -- Çıkarılacak liste

    Keyword Arguments:
        safe {bool} -- Çıkarma işlemi sırasında verilerin sırasını korur ve tekrarları kaldırmaz (default: {True})

    Returns:
        list -- Sonuç listesi
    """
    if safe:
        list2 = set(list2)
        return [x for x in list1 if x not in list2]
    else:
        return list(set(list1) - set(list2))



from distutils.core import setup
import setuptools

VERSION = "0.3.4.1"

long_description = ""
with open("README.md", "r", encoding="utf-8") as file:
    long_description = file.read()


setup(
    name='ypackage',         # How you named your package folder (MyLib)
    packages=['ypackage'],   # Chose the same as "name"
    # Start with a small number and increase it with every change you make
    version=VERSION,
    # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    license='MIT',
    # Give a short description about your library
    description='Sık kullanılan python işlemleri için hazır paket',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Yunus Emre Ak',                   # Type in your name
    author_email='yedhrab@gmail.com',      # Type in your E-Mail
    # Provide either the link to your github or to your website
    url='https://github.com/yedhrab/YPackage',
    # I explain this later on
    # download_url=f'https://github.com/yedhrab/YPackage/archive/{VERSION}.tar.gz',
    # Keywords that define your package best
    keywords=['ypackage', 'yedhrab', 'yemreak'],
    install_requires=[            # I get to this in a second

    ],
    classifiers=[
        # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Development Status :: 3 - Alpha',
        # Define that your audience are developers
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',   # Again, pick a license
        # Specify which pyhton versions that you want to support
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)
